<?php
/**
 * Comments module - Allows users to post comments on items
 *
 * @package modules
 * @copyright (C) 2002-2006 The Digital Development Foundation
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://www.xaraya.com
 *
 * @subpackage Comments Module
 * @link http://xaraya.com/index.php/release/14.html
 * @author Carl P. Corliss <rabbitt@xaraya.com>
 */
/**
 * Get the number of comments for a module item
 *
 * @author mikespub
 * @access public
 * @param integer    $modid     the id of the module that these nodes belong to
 * @param integer    $itemtype  the item type that these nodes belong to
 * @param integer    $objectid    the id of the item that these nodes belong to
 * @param integer    $status    the status of the comment: 1 - active, 2 - inactive, 3 - root node
 * @returns integer  the number of comments for the particular modid/objectid pair,
 *                   or raise an exception and return false.
 */
function comments_userapi_get_count($args)
{
    extract($args);

    $exception = false;

    if ( !isset($modid) || empty($modid) ) {
        $msg = xarML('Invalid #(1) for #(2) function #(3)() in module #(4)',
                                 'modid', 'userapi', 'get_count', 'comments');
        xarErrorSet(XAR_USER_EXCEPTION, 'BAD_PARAM',
                        new SystemException($msg));
        $exception |= true;
    }


    if ( !isset($objectid) || empty($objectid) ) {
        $msg = xarML('Invalid #(1) for #(2) function #(3)() in module #(4)',
                                 'objectid', 'userapi', 'get_count', 'comments');
        xarErrorSet(XAR_USER_EXCEPTION, 'BAD_PARAM',
                        new SystemException($msg));
        $exception |= true;
    }

    if( isset($status) && $status == 1 ) {
        $status = (int) _COM_STATUS_OFF;
    } elseif (( isset($status) && $status == 2) || !isset($status) ) {
        $status = (int) _COM_STATUS_ON;
    } elseif ( isset($status) && $status == 3 ) {
        $status = (int) _COM_STATUS_ROOT_NODE;
    } elseif ( isset($status) ) {
        $msg = xarML('Invalid #(1) for #(2) function #(3)() in module #(4)',
                                 'status', 'userapi', 'get_count', 'comments');
        xarErrorSet(XAR_USER_EXCEPTION, 'BAD_PARAM',
                        new SystemException($msg));
        $exception |= true;
    }


    if ($exception) {
        return;
    }

    $dbconn =& xarDBGetConn();
    $xartable =& xarDBGetTables();
    $ctable = &$xartable['comments_column'];

    $sql = "SELECT  COUNT($ctable[cid]) as numitems
              FROM  $xartable[comments]
             WHERE  $ctable[objectid]=? AND $ctable[modid]=?
               AND  $ctable[status]=?";
// Note: objectid is not an integer here (yet ?)
    $bindvars = array((string) $objectid, (int) $modid, (int) $status);

    if (isset($itemtype) && is_numeric($itemtype)) {
        $sql .= " AND $ctable[itemtype]=?";
        $bindvars[] = (int) $itemtype;
    }

/*    if (isset($status)) {
        $sql .= " AND $ctable[status]=?";
        $bindvars[] = (int) $status;
    }
*/
    $result =& $dbconn->Execute($sql,$bindvars);
    if (!$result)
        return;

    if ($result->EOF) {
        return 0;
    }
    list($numitems) = $result->fields;

    $result->Close();

    return $numitems;
}

?>
